#!/bin/bash
#####################################################################
# 
# set_env.bash
#
# Purpose:
# This shell script defines and set up (export) the shell ENV variables
# for the podgen package.
#
# History:  
#       Created by: Ming Chen 2023/03/24
#                   ming.chen@noaa.gov
# 
# Usage: source set_env.bash
#
#####################################################################
#
# Package home directory
pkghome=/data/home006/ming.chen/RO/RT_COSMIC2_REL1.2
export podgen_home=$pkghome

# Home diretory path of all I/O POD data 
# Default data home
export data_home=/data/data504/POD_DATA

# Directory of utility software
export utils_home=${podgen_home}/Utility

# Local archive of AIUB GNSS CODE data
export aiub_data=${data_home}/aiub

# Local archive of NASA GNSS CODE data
export nasa_data=${data_home}/nasa

# Local archive of UCAR GNSSRO LEO data
export ucar_data=${data_home}/ucar

# Daily LEO Att/podSNX data for BERNESE
export umd_data=${data_home}/umd

#Archive data
export archive_home=${data_home}

#Temporary data
export tmp_data=${data_home}/TMP
export python_path=/data/jcsda3/mchen/local/miniforge3
export rnxcmp_path=/data/data504/BSW54/tools/RNXCMP_4.1.0
export PATH=${rnxcmp_path}/bin:${python_path}/bin:$PATH
pkgsubs=${podgen_home}/DataCollect:${podgen_home}/PreProc:${podgen_home}/PostProc:
pkgsubs=${pkgsubs}:${podgen_home}/Utility:${podgen_home}/RunBern54
export PYTHONPATH=${pkgsubs}
# BENERSE package ENV
export BERN_HOME=/data/data504/BSW54
export BERN_ENV=${HOME}/LOADGPS.setvar
if [ -f ${BERN_ENV} ]; then
   source ${BERN_ENV}
else
  echo "${HOME}/LOADGPS.setvar not exist ... "
  exit 1
fi


# control variables
export _VERBOSE_=True
export _OVERWRITE_=True
