#!/bin/bash
#####################################################################
# 
# rt_batch.bash 
#
# Purpose:
# This Bash script provides an interface to real-time BERNESE processing.
# It is intended to be executed via a cron job for automated scheduling.
#
# History:  
#       Created by: Ming Chen 2026/05/5
#                   ming.chen@noaa.gov
# 
# Usage: rt_batch.bash 
# 
#
#####################################################################
#

pkghome=/home/mchen36/wrk/RT_COSMIC2_REL1.2
cd ${pkghome}
source set_env.bash

if [ -z "${podgen_home}" ]; then 
   echo "env is not set up, source the set_env.bash file first .."
   exit
fi

podgen=${podgen_home}
#python ${podgen}/rt_bern_proc.py --config-file rt_cosmic2.yaml --dry-run
python ${podgen}/rt_bern_proc.py --config-file rt_cosmic2.yaml
